class Chat:
    def __init__(self, cid, first_name, ctype):
        self.cid = cid
        self.first_name = first_name
        self.ctype = ctype


class Message:
    def __init__(self, mid, mfrom, chat, date, text):
        self.mid = mid
        self.mfrom = mfrom
        self.chat = chat
        self.date = date
        self.text = text


class Update:
    def __init__(self, uid, message):
        self.uid = uid
        self.message = message



from abc import abstractmethod

from client_corleone.models import Chat, Message, Update


class Serializer:
    @property
    def data(self):
        result = [self.serialize(datum) for datum in self._raw_data]
        return result if self.many else result[0]

    @abstractmethod
    def serialize(self, raw_data):
        pass

    def __init__(self, raw_data, many=False):
        self.many = many
        self._raw_data = raw_data if many else [raw_data]


class UpdateSerializer(Serializer):
    def serialize(self, raw_data):
        message = MessageSerializer(raw_data['message']).data
        return Update(raw_data['update_id'], message)


class MessageSerializer(Serializer):
    def serialize(self, raw_data):
        chat = ChatSerializer(raw_data['chat']).data
        return Message(
            raw_data['message_id'], raw_data['from'],
            chat, raw_data['date'], raw_data['text']
        )


class ChatSerializer(Serializer):
    def serialize(self, raw_data):
        return Chat(cid=raw_data['id'], first_name=raw_data['first_name'], ctype=raw_data['type'])



import logging
import requests
import time

from client_corleone.serializers import UpdateSerializer


logger = logging.getLogger('client')


class BotClient:
    def __init__(self, token, max_retries=None):
        self.bot_url = f'https://api.telegram.org/bot{token}'
        self.max_retries = max_retries

    def updates(self):
        offset = 0
        retries = self.max_retries

        while True:
            try:
                raw_updates = requests.get(f'{self.bot_url}/getUpdates?offset={offset}').json()
                if not raw_updates.get('ok'):
                    logger.error('Updates NOK')
                    raise ConnectionError('Updates NOK')
            except ConnectionError:
                logger.error('Connection error')
                if retries:
                    retries -= 1
                    # Waits: 1s, 2s, 4s, 8s...
                    time.sleep((self.max_retries - retries) ** 2)
                    continue
                return

            retries = self.max_retries

            updates = UpdateSerializer(raw_updates['result'], many=True).data

            for update in updates:
                yield update
                offset = update.uid + 1

    def send_text(self, chat_id, text):
        requests.post(f'{self.bot_url}/sendMessage', data={'chat_id': chat_id, 'text': text})



from setuptools import setup, find_packages

import client_corleone

setup(
    name='client_corleone',
    version=client_corleone.__version__,
    packages=find_packages(),
    author='Luis Sieira',
    author_email='luis.sieira@spacebar.fr',
    description='A python experimental client for Telegram bot API you can\'t refuse',
    long_description=open('README.md').read(),
    install_requires=['requests'],
    include_package_data=True,
    url='https://github.com/sieira/client_corleone',
    license='BSD',
)
